@@ -3,7 +3,6 @@ from __future__ import annotations
 
 import bisect
 import concurrent.futures
-import gc
 import itertools
 import json
 import logging
@@ -35,16 +34,6 @@ from upathlib.serializer import (
 logger = logging.getLogger(__name__)
 
 
-@contextmanager
-def no_gc():
-    isgc = gc.isenabled()
-    if isgc:
-        gc.disable()
-    yield
-    if isgc:
-        gc.enable()
-
-
 class Dumper:
     def __init__(self, max_workers: int = 8):
         assert 0 < max_workers < 20
@@ -100,85 +89,6 @@ class Dumper:
         self._task_file_data = {}
 
 
-class ConcurrentIterStat:
-    def __init__(self, data_info=None, iter_info=None):
-        self._data_info = data_info
-        self._iter_info = iter_info
-
-    def __repr__(self) -> str:
-        return "{}({}, {}, {})".format(
-            self.__class__.__name__,
-            self.n_files_total,
-            self.n_files_claimed,
-            self.n_files_finished
-        )
-
-    def __str__(self) -> str:
-        return self.__repr__()
-
-    @property
-    def time_started(self) -> Optional[datetime]:
-        if self._iter_info is None:
-            return None
-        return datetime.fromisoformat(
-            self._iter_info[0]['time_started'])
-
-    @property
-    def time_finished(self) -> Optional[datetime]:
-        if self._iter_info is None:
-            return None
-        if len(self._iter_info) < len(self._data_info):
-            return None
-        tt = None
-        for z in self._iter_info:
-            t = z.get('time_finished')
-            if t is None:
-                return None
-            t = datetime.fromisoformat(t)
-            if tt is None or t > tt:
-                tt = t
-        return tt
-
-    @property
-    def finished(self) -> bool:
-        if self._iter_info is None:
-            return False
-        if len(self._iter_info) < len(self._data_info):
-            return False
-        for z in self._iter_info:
-            if 'time_finished' not in z:
-                return False
-        return True
-
-    @property
-    def n_files_total(self) -> int:
-        return len(self._data_info)
-
-    @property
-    def n_files_claimed(self) -> int:
-        if self._iter_info is None:
-            return 0
-        return len(self._iter_info)
-
-    @property
-    def n_files_finished(self) -> int:
-        if self._iter_info is None:
-            return 0
-        n = 0
-        for z in self._iter_info:
-            if 'time_finished' in z:
-                n += 1
-        return n
-
-    @property
-    def status(self) -> str:
-        if self.time_started is None:
-            return 'NOT STARTED'
-        if self.finished:
-            return 'FINISHED'
-        return 'IN PROGRESS'
-
-
 class Biglist(Sequence):
     '''
     Data access is optimized for iteration, whereas random access
@@ -196,10 +106,10 @@ class Biglist(Sequence):
                                 ):
         good = string.ascii_letters + string.digits + '-_'
         assert all(n in good for n in name)
-        if name.replace('-', '_') in cls.registered_storage_formats:
+        if name.replace('_', '-') in cls.registered_storage_formats:
             if not overwrite:
                 raise ValueError(f"serializer '{name}' is already registered")
-        name = name.replace('-', '_')
+        name = name.replace('_', '-')
         cls.registered_storage_formats[name] = serializer
 
     @classmethod
@@ -213,16 +123,16 @@ class Biglist(Sequence):
 
     @classmethod
     def dump_data_file(cls, path: Upath, data: list):
-        serializer = cls.registered_storage_formats[path.suffix.lstrip('.')]
+        serializer = cls.registered_storage_formats[path.suffix.lstrip('.').replace('_', '-')]
         data = [cls.pre_serialize(v) for v in data]
         path.write_bytes(serializer.serialize(data))
 
     @classmethod
     def load_data_file(cls, path: Upath):
-        deserializer = cls.registered_storage_formats[path.suffix.lstrip('.')]
-        with no_gc():
-            z = deserializer.deserialize(path.read_bytes())
-            return [cls.post_deserialize(v) for v in z]
+        deserializer = cls.registered_storage_formats[path.suffix.lstrip('.').replace('_', '-')]
+        data = path.read_bytes()
+        z = deserializer.deserialize(data)
+        return [cls.post_deserialize(v) for v in z]
 
     @classmethod
     @contextmanager
@@ -321,12 +231,10 @@ class Biglist(Sequence):
 
         if storage_format is None:
             storage_format = cls.DEFAULT_STORAGE_FORMAT
-            obj.info['storage_format'] = storage_format
-        else:
-            if storage_format.replace('-', '_') not in cls.registered_storage_formats:
-                raise ValueError(
-                    f"invalid value of `storage_format`: '{storage_format}'")
-            obj.info['storage_format'] = storage_format.replace('-', '_')
+        if storage_format.replace('_', '-') not in cls.registered_storage_formats:
+            raise ValueError(
+                f"invalid value of `storage_format`: '{storage_format}'")
+        obj.info['storage_format'] = storage_format.replace('_', '-')
         obj.info['storage_version'] = 0
         # version 0 designator introduced on 2022/3/8
         obj._info_file.write_json(obj.info, overwrite=False)
@@ -380,7 +288,11 @@ class Biglist(Sequence):
 
     @property
     def storage_format(self) -> str:
-        return self.info['storage_format']
+        return self.info['storage_format'].replace('_', '-')
+
+    @property
+    def datafile_ext(self) -> str:
+        return self.storage_format.replace('-', '_')
 
     @property
     def storage_version(self) -> int:
@@ -553,7 +465,7 @@ class Biglist(Sequence):
         datafiles = self.get_data_files()
         return [
             self.file_view(self._data_dir / f)
-            for f, l in datafiles
+            for f, _ in datafiles
         ]
 
     def _flush(self, *, wait: bool = False, concurrent: bool = True):
@@ -575,7 +487,7 @@ class Biglist(Sequence):
             data_files = self.get_data_files(lazy=not concurrent)
 
             while True:
-                data_file = f'{uuid4()}.{self.storage_format}'
+                data_file = f"{uuid4()}.{self.datafile_ext}"
                 # TODO: this could be inefficient if there are many files.
                 if not any((data_file == v[0] for v in data_files)):
                     break
@@ -669,59 +581,44 @@ class Biglist(Sequence):
         by exactly one worker.
         '''
         task_id = datetime.utcnow().isoformat()
-        self._concurrent_iter_info_file(task_id).write_json([], overwrite=True)
+        self._concurrent_iter_info_file(task_id).write_json(
+            {'n_files_claimed': 0}, overwrite=True)
         return task_id
 
     def concurrent_iter(self, task_id: str, *, reader_id: str = None) -> Iterator:
         if not reader_id and isinstance(self.path, LocalUpath):
             reader_id = multiprocessing.current_process().name
         datafiles = self.get_data_files()
-        file_idx = None
-        file_name = None
         while True:
             ff = self._concurrent_iter_info_file(task_id)
             with self._lockfile(ff.with_suffix('.json.lock')):
                 iter_info = ff.read_json()
-                if file_idx is not None:
-                    # This is the file processed in last round.
-                    # Update its record.
-                    z = iter_info[file_idx]
-                    assert z['file_name'] == file_name
-                    assert z['reader_id'] == reader_id
-                    iter_info[file_idx]['time_finished'] = str(
-                        datetime.utcnow())
-
-                if len(iter_info) >= len(datafiles):
+                n_files_claimed = iter_info['n_files_claimed']
+                if n_files_claimed >= len(datafiles):
                     # No more date files to process.
-                    ff.write_json(iter_info, overwrite=True)
                     break
 
-                # Get the next file to process.
-                # Claim its spot in the book-keeping file.
-                file_idx = len(iter_info)
-                file_name = datafiles[file_idx][0]
-                iter_info.append({
-                    'file_name': file_name,
-                    'reader_id': reader_id,
-                    'time_started': str(datetime.utcnow()),
-                })
+                iter_info['n_files_claimed'] = n_files_claimed + 1
                 ff.write_json(iter_info, overwrite=True)
+                file_name = datafiles[n_files_claimed][0]
+                fv = self.file_view(self._data_dir / file_name)
+                # This does not actually read the file.
+                # TODO: make this read the file here?
 
-            fv = self.file_view(self._data_dir / file_name)
             logger.debug('yielding data of file "%s"', file_name)
             yield from fv.data  # this is the data list contained in the data file
 
-    def concurrent_iter_stat(self, task_id: str, lazy: bool = True) -> ConcurrentIterStat:
+    def concurrent_iter_done(self, task_id: str, lazy: bool = True) -> Optional[bool]:
         try:
             iter_info = self._concurrent_iter_info_file(task_id).read_json()
         except FileNotFoundError:
-            return ConcurrentIterStat()
+            return None
         datafiles = self.get_data_files(lazy=lazy)
         # Usually, this method is called by a "controller" node to
         # check the status of iteration of the biglist by multiple worker nodes.
         # In the meantime, the biglist remain unchanged, hence
         # `get_data_files` can use the lazy mode.
-        return ConcurrentIterStat(datafiles, iter_info)
+        return iter_info['n_files_claimed'] >= len(datafiles)
 
 
 class FileView(Sequence):
